from unittest import TestCase
from riki_usermanager import User
from faker import Faker


class TestUser(TestCase):
    def setUp(self) -> None:
        faker = Faker()
        self.username = faker.profile()['username']

        self.user = User(self.username, faker.password(), active=False)
        return super().setUp()

    def test_User__is_authenticated(self):
        self.assertFalse(self.user.is_authenticated())

    def test_User__is_active(self):
        self.assertTrue(self.user.is_active())

    def test_User__is_anonymous(self):
        self.assertFalse(self.user.is_anonymous())

    def test_User__get_id(self):
        self.assertEqual(self.user.get_id(), self.username)

    def test_User__from_dict(self):
        user_dict = {
            "username": self.user.username,
            "password": self.user.password,
            "hash": self.user.hash,
            "active": self.user.active,
            "roles": " ".join(self.user.roles),
            "authentication_method": self.user.authentication_method,
            "authenticated": self.user.authenticated
        }

        self.assertEqual(User.from_dict(user_dict), self.user)




import binascii
import hashlib
import os
import sqlite3
from typing import Literal, Union

from .User import AuthMethodEnum, User


class UserManager:
    """A very simple User Manager, that manages `User` objects and writes them to database"""

    def __init__(self, db: sqlite3.Connection):
        """Create UserManager object

        # Args:
            - db (sqlite3.Connection): preexisting sqlite3 connection object
        """
        self.db = db
        self.db.row_factory = sqlite3.Row

    def login(self, username: str, password: str) -> Union[User, Literal[False]]:
        """Logins in a user after username and password have been validated

        Args:
            username (String): username

        Returns:
            bool: ``True`` on success, else False
        """
        user = self.get_user(username)

        # user does not exist
        if user is None:
            return False

        if not self.check_password(user, password):
            return False

        return user

    def logout(self, user: User) -> bool:
        """Logs out the current user

        Returns:
            bool: ``True`` on success, else False
        """
        user.authenticated = False
        return self.update(user)

    def register(self, user: 'User') -> Union['User', Literal[False]]:
        """Creates a new user and authenticates a new user after username and password are validated

        Args:
            user (User): User object


        Returns:
            bool: ``True`` on success, else False
        """
        return self.add_user(user)

    def unregister(self, user: User) -> bool:
        """Deletes current user's profile

        Returns:
            bool: ``True`` on success, else False
        """
        return self.delete_user(user.get_id())

    def add_user(self, user: 'User') -> Union[User, Literal[False]]:
        """Creates new user in the database

        Args:
            user (User): User object

        Raises:
            NotImplementedError: if authentication method is not implemented

        Returns:
            bool: ``True`` on success, else False
        """
        cur = self.db.execute(
            'select * from users where username = ?',
            (user.username,)
        )

        result = cur.fetchall()

        # user already exists
        if result:
            return False

        if user.authentication_method == AuthMethodEnum.HASH:
            user.hash = make_salted_hash(user.password)

        self.db.execute(
            'insert into users (username, active, authentication_method, password, hash, authenticated, roles) values (?, ?, ?, ?, ?, ?, ?)',
            (
                # The following are to be stored in the sqlite database.
                # Any booleans will have to be stored as integers since
                # sqlite does not support the boolean datatype.

                # username is intended to be store as text and will be the key that
                # all entries are indexed by.
                user.username,

                # active will mark whether the user account is active.
                # It will be stored in sqlite as an int
                user.active,

                # Our authentication methods(not to be confused with Python methods)
                # are identified by an int and that will be how we store them in the
                # sqlite database.
                user.authentication_method.value,

                # This is the user's password.  It'll be stored as a text value.
                user.password,

                # If the password is hased, this will hold the result.
                # Also stored as text.
                user.hash,
                user.authenticated,
                " ".join(user.roles)
            )
        )
        self.db.commit()
        return user

    def get_user(self, username: str) -> Union['User', None]:
        """Get `User` from the database

        # Args:
            - name (str): users name
        # Returns:
            - User | None: User object if user with the given username is found, otherwise nothing is returned.
        """
        cur = self.db.execute(
            'select * from users where username = ?',
            (username,)
        )
        user = cur.fetchone()
        if not user:
            return None
        return User.from_dict(user)

    def delete_user(self, username: str) -> bool:
        """Deletes user from the database

        Args:
            name (str): users username

        Returns:
            bool: True if delete was successful, otherwise False
        """
        # https://stackoverflow.com/questions/13313694/how-do-i-determine-if-the-row-has-been-inserted
        try:
            self.db.execute(
                'delete from users where username = ?',
                (username,)
            )
            self.db.commit()
            return True
        except sqlite3.Error:
            return False

    def update(self, user: 'User') -> bool:
        """Update user from userdata dictionary

        Args:
            name (str): users username
            userdata (Dict[str, Any]):  dictionary of user data

        Returns:
            bool: True if delete was successful, otherwise False
        """
        try:
            self.db.execute(
                'update users set active = ?, password = ?, authenticated = ? where username = ?',
                (
                    user.active,
                    user.password,
                    user.authenticated,
                    user.username
                )
            )

            self.db.commit()
            return True
        except sqlite3.Error:
            return False

    @staticmethod
    def check_password(user: User, password: str) -> bool:
        """Check if user password matches the password in the database

        Args:
            password (str): user password

        Returns:
            bool: did password match
        """

        authentication_method = user.authentication_method
        result = False

        if authentication_method is AuthMethodEnum.HASH.value:
            result = check_hashed_password(password, user.password)

        elif authentication_method is AuthMethodEnum.CLEARTEXT.value:
            result = (user.password == password)

        return result


def make_salted_hash(password, salt=None):
    if not salt:
        salt = os.urandom(64)
    d = hashlib.sha512()
    d.update(salt[:32])
    d.update(password)
    d.update(salt[32:])
    return str(binascii.hexlify(salt)) + d.hexdigest()


def check_hashed_password(password, salted_hash):
    salt = binascii.unhexlify(salted_hash[:128])
    return make_salted_hash(password, salt) == salted_hash



import unittest
import sqlite3
from riki_usermanager import AuthMethodEnum, UserManager, User
from faker import Faker
from pathlib import Path


class TestUserManager(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        new_db = sqlite3.connect(':memory:')  # create a memory database
        new_db.executescript(Path('schema.sql').read_text())
        cls.faker = Faker()

        profile = cls.faker.profile()

        new_db.execute(
            'insert into users (username, active, authentication_method, password, hash, authenticated, roles) values (?, ?, ?, ?, ?, ?, ?)',
            (profile['username'], True, 0,
             cls.faker.password(), '', False, '', )
        )

        cls.existing_profile = profile
        cls.user_manager = UserManager(new_db)

    def test_add_new_user(self):
        profile = self.faker.profile()
        password = self.faker.password()

        user = User(profile['username'], password=password,
                    active=True, authentication_method=AuthMethodEnum.CLEARTEXT)

        self.assertTrue(self.user_manager.add_user(user))
        self.assertEqual(user.username, profile['username'])
        self.assertEqual(user.authentication_method, AuthMethodEnum.CLEARTEXT)
        self.assertFalse(self.user_manager.add_user(user))

    def test_delete_user(self):
        profile = self.faker.profile()
        password = self.faker.password()

        user = User(profile['username'], password=password,
                    active=True, authentication_method=AuthMethodEnum.CLEARTEXT)

        self.assertTrue(self.user_manager.add_user(user))
        self.assertTrue(self.user_manager.delete_user(user.username))

    def test_get_existing_user(self):
        profile = self.existing_profile
        user = self.user_manager.get_user(profile['username'])

        self.assertEqual(user.username, profile['username'])

    def test_get_nonexisting_user(self):
        profile = self.faker.profile()
        user = self.user_manager.get_user(profile['username'])

        self.assertIsNone(user)

    def test_update_user(self):
        profile = self.faker.profile()
        new_profile = self.faker.profile()
        password = self.faker.password()
        new_password = self.faker.password()

        user = User(profile['username'], password=password,
                    active=True, authentication_method=AuthMethodEnum.CLEARTEXT)
        updated_user = User(profile['username'], password=password,
                            active=False, authentication_method=AuthMethodEnum.CLEARTEXT)
        self.user_manager.add_user(user)

        self.assertTrue(self.user_manager.update(updated_user))

        user = self.user_manager.get_user(profile['username'])
        self.assertFalse(user.active)



from dataclasses import dataclass, field
from enum import Enum
from typing import Any, Dict, List


class AuthMethodEnum(Enum):
    """Enum that stores supported authentication methods
    """
    CLEARTEXT = 0
    HASH = 1


@dataclass
class User:
    """Represents User entry in the sqlite3 database

    Variables: 
        * username (str): The user's name.  Used as the primary key in the database.
        * password (str): The user's password.  Stored as text in sqlite.
        * roles (str): The roles a user has.  It's a list of string, but will be
            stored as a single text value in sqlite.
        * authentication_method (int): Used to reference an authentication
            method by number.
        * authenticated (bool): Flag for whether a user has been authenticated.
            Stored in sqlite as an int.
        * hash (str): Stored result if password has been hashed.
        * anonymous (bool): Flag for anonymous users.  Since a registered user is 
            not anonymous, this is not stored in sqlite.
    """
    username: str
    password: str

    active: bool = field(default=True)
    roles: List[str] = field(default_factory=list)
    authentication_method: AuthMethodEnum = field(
        default=AuthMethodEnum.CLEARTEXT)
    authenticated: bool = field(default=False)
    hash: str = field(default="")
    anonymous: bool = field(default=False)

    def is_authenticated(self):
        """Returns whether the User is authenticated.

        Args:
        self (int): The current instance of User

        Returns:
        bool:authentication state

        """
        return self.authenticated

    def is_active(self):
        """Returns whether the User is active. Required by flask-login.

        Args:
        self (int): The current instance of User

        Returns:
        bool:active state

        """
        return self.is_active

    def is_anonymous(self):
        """Returns whether the User is anonymous.  In this case, all users are not.

        Args:
        self (int): The current instance of User

        Returns:
        bool:anonymous state


        """
        return self.anonymous

    def get_id(self):
        """Returns the username of a user. Required by flask-login.

        Args:
        self (int): The current instance of User

        Returns:
        str:username

        """
        return self.username

    @staticmethod
    def from_dict(user: Dict[str, Any]) -> 'User':
        """converts array of sql data into dictionary

        Args:
            data (List[str]): sql array of data

        Returns:
            Dict[str, Any]: Dictionary of values
        """

        roles = user["roles"].split()
        return User(username=user["username"],
                    password=user["password"],
                    hash=user["hash"],
                    active=user["active"],
                    roles=roles,
                    authentication_method=user["authentication_method"],
                    authenticated=user["authenticated"])



from setuptools import find_packages, setup


setup(
    name="Riki Usermanager",
    version="1.0.1",
    description="User Manager utilities for the Riki",
    author="Arseny Poga",
    author_email="pogaa1@nku.edu",
    packages=find_packages(),
)
